@@ -7,11 +7,19 @@ test_that("MCMC results for Gaussian model are correct",{
     sd_y = uniform(1, 0, 10), 
     sd_level = uniform(1, 0, 10))
   
-  expect_equal(run_mcmc(model_bssm, n_iter = 10, seed = 1)[-13], 
-    run_mcmc(model_bssm, n_iter = 10, seed = 1)[-13])
-  
   expect_error(mcmc_bsm <- run_mcmc(model_bssm, n_iter = 50, seed = 1), NA)
   
+  expect_equal(run_mcmc(model_bssm, n_iter = 10, seed = 1)[-14], 
+    run_mcmc(model_bssm, n_iter = 10, seed = 1)[-14])
+  expect_equal(run_mcmc(model_bssm, n_iter = 10, seed = 1, type = "summary")[-15], 
+    run_mcmc(model_bssm, n_iter = 10, seed = 1, type = "summary")[-15])
+  expect_equal(run_mcmc(model_bssm, n_iter = 10, seed = 1, type = "theta")[-13], 
+    run_mcmc(model_bssm, n_iter = 10, seed = 1, type = "theta")[-13])
+  expect_equal(run_mcmc(model_bssm, n_iter = 10, seed = 1, type = "theta")$theta, 
+    run_mcmc(model_bssm, n_iter = 10, seed = 1, type = "summary")$theta)
+  expect_equal(run_mcmc(model_bssm, n_iter = 10, seed = 1, type = "theta")$acceptance_rate, 
+    run_mcmc(model_bssm, n_iter = 10, seed = 1, type = "summary")$acceptance_rate)
+  
   expect_gt(mcmc_bsm$acceptance_rate, 0)
   expect_gte(min(mcmc_bsm$theta), 0)
   expect_lt(max(mcmc_bsm$theta), Inf)
@@ -25,11 +33,15 @@ test_that("MCMC results for Poisson model are correct",{
   model_bssm <- ng_bsm(rpois(10, exp(0.2) * (2:11)), P1 = diag(2, 2), sd_slope = 0,
     sd_level = uniform(2, 0, 10), u = 2:11, distribution = "poisson")
   
-  expect_equal(run_mcmc(model_bssm, n_iter = 10, nsim_states = 5, seed = 1)[-13], 
-    run_mcmc(model_bssm, n_iter = 10, nsim_states = 5, seed = 1)[-13])
-  
   expect_error(mcmc_poisson <- run_mcmc(model_bssm, n_iter = 10, nsim_states = 5, seed = 42), NA)
   
+  expect_equal(run_mcmc(model_bssm, n_iter = 10, seed = 1, nsim_states = 5)[-14], 
+    run_mcmc(model_bssm, n_iter = 10, seed = 1, nsim_states = 5)[-14])
+  expect_equal(run_mcmc(model_bssm, n_iter = 10, seed = 1, type = "summary", nsim_states = 5)[-15], 
+    run_mcmc(model_bssm, n_iter = 10, seed = 1, type = "summary", nsim_states = 5)[-15])
+  expect_equal(run_mcmc(model_bssm, n_iter = 10, seed = 1, type = "theta", nsim_states = 5)[-13], 
+    run_mcmc(model_bssm, n_iter = 10, seed = 1, type = "theta", nsim_states = 5)[-13])
+
   expect_gt(mcmc_poisson$acceptance_rate, 0)
   expect_gte(min(mcmc_poisson$theta), 0)
   expect_lt(max(mcmc_poisson$theta), Inf)
@@ -44,18 +56,18 @@ test_that("MCMC results for SV model using IS-correction are correct",{
     sd_ar = halfnormal(1, 5), sigma = halfnormal(1, 2)), NA)
   
   expect_equal(run_mcmc(model_bssm, n_iter = 100, nsim_states = 10,
-    method = "is2", seed = 1)[-14], 
-    run_mcmc(model_bssm, n_iter = 100, nsim_states = 10, method = "is2", seed = 1)[-14])
+    method = "is2", seed = 1)[-15], 
+    run_mcmc(model_bssm, n_iter = 100, nsim_states = 10, method = "is2", seed = 1)[-15])
   
   expect_equal(run_mcmc(model_bssm, n_iter = 100, nsim_states = 10,
-    method = "is2", seed = 1, simulation_method = "psi")[-14], 
+    method = "is2", seed = 1, simulation_method = "psi")[-15], 
     run_mcmc(model_bssm, n_iter = 100, nsim_states = 10, 
-      method = "is2", seed = 1, simulation_method = "psi")[-14])
+      method = "is2", seed = 1, simulation_method = "psi")[-15])
   
   expect_equal(run_mcmc(model_bssm, n_iter = 100, nsim_states = 10,
-    method = "is2", seed = 1, simulation_method = "bsf")[-14], 
+    method = "is2", seed = 1, simulation_method = "bsf")[-15], 
     run_mcmc(model_bssm, n_iter = 100, nsim_states = 10, 
-      method = "is2", seed = 1, simulation_method = "bsf")[-14])
+      method = "is2", seed = 1, simulation_method = "bsf")[-15])
   
   expect_error(mcmc_sv <- run_mcmc(model_bssm, n_iter = 100, nsim_states = 10,
     method = "is2", seed = 1, simulation_method = "bsf"), NA)
